@@ -1,4 +1,4 @@
-# Copyright 2002, 2003 Ben Escoto
+# Copyright 2002, 2003, 2004 Ben Escoto
 #
 # This file is part of rdiff-backup.
 #
@@ -20,7 +20,7 @@
 """Start (and end) here - read arguments, set global settings, etc."""
 
 from __future__ import generators
-import getopt, sys, re, os
+import getopt, sys, re, os, cStringIO
 from log import Log, LoggerError, ErrorLog
 import Globals, Time, SetConnections, selection, robust, rpath, \
 	   manage, backup, connection, restore, FilenameMapping, \
@@ -36,6 +36,7 @@ user_mapping_filename, group_mapping_filename = None, None
 # These are global because they are set while we are trying to figure
 # whether to restore or to backup
 restore_root, restore_index, restore_root_set = None, None, 0
+return_val = None # Set to cause exit code to be specified value
 
 def parse_cmdlineoptions(arglist):
 	"""Parse argument list and set global preferences"""
@@ -47,20 +48,24 @@ def parse_cmdlineoptions(arglist):
 		try: return open(filename, "r")
 		except IOError: Log.FatalError("Error opening file %s" % filename)
 
+	def normalize_path(path):
+		"""Used below to normalize the security paths before setting"""
+		return rpath.RPath(Globals.local_connection, path).normalize().path
+
 	try: optlist, args = getopt.getopt(arglist, "blr:sv:V",
 		 ["backup-mode", "calculate-average", "check-destination-dir",
-		  "current-time=", "exclude=", "exclude-device-files",
-		  "exclude-filelist=", "exclude-filelist-stdin",
-		  "exclude-globbing-filelist=", "exclude-mirror=",
-		  "exclude-other-filesystems", "exclude-regexp=",
-		  "exclude-special-files", "force", "group-mapping-file=",
-		  "include=", "include-filelist=", "include-filelist-stdin",
-		  "include-globbing-filelist=", "include-regexp=",
-		  "list-at-time=", "list-changed-since=", "list-increments",
-		  "list-increment-sizes", "never-drop-acls", "no-acls",
-		  "no-compare-inode", "no-compression",
-		  "no-compression-regexp=", "no-eas", "no-file-statistics",
-		  "no-hard-links", "null-separator",
+		  "compare", "compare-at-time=", "current-time=", "exclude=",
+		  "exclude-device-files", "exclude-filelist=",
+		  "exclude-filelist-stdin", "exclude-globbing-filelist=",
+		  "exclude-mirror=", "exclude-other-filesystems",
+		  "exclude-regexp=", "exclude-special-files", "force",
+		  "group-mapping-file=", "include=", "include-filelist=",
+		  "include-filelist-stdin", "include-globbing-filelist=",
+		  "include-regexp=", "list-at-time=", "list-changed-since=",
+		  "list-increments", "list-increment-sizes",
+		  "never-drop-acls", "no-acls", "no-compare-inode",
+		  "no-compression", "no-compression-regexp=", "no-eas",
+		  "no-file-statistics", "no-hard-links", "null-separator",
 		  "override-chars-to-quote=", "parsable-output",
 		  "print-statistics", "remote-cmd=", "remote-schema=",
 		  "remove-older-than=", "restore-as-of=", "restrict=",
@@ -74,6 +79,10 @@ def parse_cmdlineoptions(arglist):
 		if opt == "-b" or opt == "--backup-mode": action = "backup"
 		elif opt == "--calculate-average": action = "calculate-average"
 		elif opt == "--check-destination-dir": action = "check-destination-dir"
+		elif opt == "--compare" or opt == "--compare-at-time":
+			action = "compare"
+			if opt == "--compare": restore_timestr = "now"
+			else: restore_timestr = arg
 		elif opt == "--current-time":
 			Globals.set_integer('current_time', arg)
 		elif opt == "--exclude": select_opts.append((opt, arg))
@@ -131,13 +140,15 @@ def parse_cmdlineoptions(arglist):
 		elif opt == "--remove-older-than":
 			remove_older_than_string = arg
 			action = "remove-older-than"
-		elif opt == "--restrict": Globals.restrict_path = arg
+		elif opt == "--no-resource-forks":
+			Globals.set('resource_forks_active', 0)
+		elif opt == "--restrict": Globals.restrict_path = normalize_path(arg)
 		elif opt == "--restrict-read-only":
 			Globals.security_level = "read-only"
-			Globals.restrict_path = arg
+			Globals.restrict_path = normalize_path(arg)
 		elif opt == "--restrict-update-only":
 			Globals.security_level = "update-only"
-			Globals.restrict_path = arg
+			Globals.restrict_path = normalize_path(arg)
 		elif opt == "-s" or opt == "--server":
 			action = "server"
 			Globals.server = 1
@@ -159,7 +170,7 @@ def check_action():
 					   1: ['list-increments', 'list-increment-sizes',
 						   'remove-older-than', 'list-at-time',
 						   'list-changed-since', 'check-destination-dir'],
-					   2: ['backup', 'restore', 'restore-as-of']}
+					   2: ['backup', 'restore', 'restore-as-of', 'compare']}
 	l = len(args)
 	if l == 0 and action not in arg_action_dict[l]:
 		commandline_error("No arguments given")
@@ -182,7 +193,7 @@ def final_set_action(rps):
 def commandline_error(message):
 	sys.stderr.write("Error: %s\n" % message)
 	sys.stderr.write("See the rdiff-backup manual page for instructions\n")
-	sys.exit(1)
+	sys.exit(2)
 
 def misc_setup(rps):
 	"""Set default change ownership flag, umask, relay regexps"""
@@ -216,16 +227,17 @@ def take_action(rps):
 		connection.PipeConnection(sys.stdin, sys.stdout).Server()
 		sys.exit(0)
 	elif action == "backup": Backup(rps[0], rps[1])
-	elif action == "restore": Restore(*rps)
-	elif action == "restore-as-of": Restore(rps[0], rps[1], 1)
-	elif action == "test-server": SetConnections.TestConnections()
+	elif action == "calculate-average": CalculateAverage(rps)
+	elif action == "check-destination-dir": CheckDest(rps[0])
+	elif action == "compare": Compare(*rps)
 	elif action == "list-at-time": ListAtTime(rps[0])
 	elif action == "list-changed-since": ListChangedSince(rps[0])
 	elif action == "list-increments": ListIncrements(rps[0])
 	elif action == 'list-increment-sizes': ListIncrementSizes(rps[0])
 	elif action == "remove-older-than": RemoveOlderThan(rps[0])
-	elif action == "calculate-average": CalculateAverage(rps)
-	elif action == "check-destination-dir": CheckDest(rps[0])
+	elif action == "restore": Restore(*rps)
+	elif action == "restore-as-of": Restore(rps[0], rps[1], 1)
+	elif action == "test-server": SetConnections.TestConnections()
 	else: raise AssertionError("Unknown action " + action)
 
 def cleanup():
@@ -246,6 +258,7 @@ def Main(arglist):
 	misc_setup(rps)
 	take_action(rps)
 	cleanup()
+	if return_val is not None: sys.exit(return_val)
 
 
 def Backup(rpin, rpout):
@@ -383,6 +396,8 @@ def backup_set_fs_globals(rpin, rpout):
 	update_triple(src_fsa.resource_forks, dest_fsa.resource_forks,
 				  ('resource_forks_active', 'resource_forks_write',
 				   'resource_forks_conn'))
+	update_triple(src_fsa.carbonfile, dest_fsa.carbonfile,
+				  ('carbonfile_active', 'carbonfile_write', 'carbonfile_conn'))
 	if Globals.never_drop_acls and not Globals.acls_active:
 		Log.FatalError("--never-drop-acls specified, but ACL support\n"
 					   "disabled on destination filesystem")
@@ -487,6 +502,8 @@ def restore_set_fs_globals(target):
 	update_triple(mirror_fsa.resource_forks, target_fsa.resource_forks,
 				  ('resource_forks_active', 'resource_forks_write',
 				   'resource_forks_conn'))
+	update_triple(mirror_fsa.carbonfile, target_fsa.carbonfile,
+				  ('carbonfile_active', 'carbonfile_write', 'carbonfile_conn'))
 	if Globals.never_drop_acls and not Globals.acls_active:
 		Log.FatalError("--never-drop-acls specified, but ACL support\n"
 					   "disabled on destination filesystem")
@@ -501,16 +518,26 @@ def restore_set_fs_globals(target):
 		else: SetConnections.UpdateGlobal('chars_to_quote', "")
 
 def restore_set_select(mirror_rp, target):
-	"""Set the selection iterator on mirror side from command line args
+	"""Set the selection iterator on both side from command line args
 
-	Here we set the selector on the mirror side, because that is where
-	we will be filtering, but the pathnames are relative to the target
-	directory.
+	We must set both sides because restore filtering is different from
+	select filtering.  For instance, if a file is excluded it should
+	not be deleted from the target directory.
+
+	The StringIO stuff is because filelists need to be read and then
+	duplicated, because we need two copies of them now.
 
 	"""
+	def fp2string(fp):
+		buf = fp.read()
+		assert not fp.close()
+		return buf
+	select_data = map(fp2string, select_files)
 	if select_opts: 
 		mirror_rp.conn.restore.MirrorStruct.set_mirror_select(
-			target, select_opts, *select_files)
+			target, select_opts, *map(cStringIO.StringIO, select_data))
+		target.conn.restore.TargetStruct.set_target_select(
+			target, select_opts, *map(cStringIO.StringIO, select_data))
 
 def restore_start_log(rpin, target, time):
 	"""Open restore log file, log initial message"""
@@ -579,7 +606,6 @@ def restore_set_root(rpin):
 		parent_dir = rpath.RPath(rpin.conn, "/".join(pathcomps[:i]))
 		if (parent_dir.isdir() and
 			"rdiff-backup-data" in parent_dir.listdir()): break
-		print "##########", parent_dir.path, Globals.restrict_path
 		if parent_dir.path == rpin.conn.Globals.get('restrict_path'):
 			return None
 		i = i-1
@@ -607,7 +633,7 @@ def restore_set_root(rpin):
 
 def ListIncrements(rp):
 	"""Print out a summary of the increments and their times"""
-	assert restore_set_root(rp)
+	rp = require_root_set(rp)
 	restore_check_backup_dir(restore_root)
 	mirror_rp = restore_root.new_index(restore_index)
 	inc_rpath = Globals.rbdir.append_path('increments', restore_index)
@@ -617,11 +643,60 @@ def ListIncrements(rp):
 		print manage.describe_incs_parsable(incs, mirror_time, mirror_rp)
 	else: print manage.describe_incs_human(incs, mirror_time, mirror_rp)
 
+def require_root_set(rp):
+	"""Make sure rp is or is in a valid rdiff-backup dest directory.
+
+	Also initializes fs_abilities and quoting and return quoted rp if
+	necessary.
+
+	"""
+	if not restore_set_root(rp):
+		Log.FatalError(("Bad directory %s.\n" % (rp.path,)) +
+		  "It doesn't appear to be an rdiff-backup destination dir")
+	single_set_fs_globals(Globals.rbdir)
+	if Globals.chars_to_quote: return restore_init_quoting(rp)
+	else: return rp
+	
+def single_set_fs_globals(rbdir):
+	"""Use fs_abilities to set globals that depend on filesystem.
+
+	This is appropriate for listing increments, or any other operation
+	that depends only on the one file system.
+
+	"""
+	def update_triple(fsa_support, attr_triple):
+		"""Update global settings based on fsa result"""
+		active_attr, write_attr, conn_attr = attr_triple
+		if Globals.get(active_attr) == 0: return # don't override 0
+		for attr in attr_triple: SetConnections.UpdateGlobal(attr, None)
+		if not fsa_support: return
+		SetConnections.UpdateGlobal(active_attr, 1)
+		SetConnections.UpdateGlobal(write_attr, 1)
+		rbdir.conn.Globals.set_local(conn_attr, 1)
+
+	fsa = rbdir.conn.fs_abilities.get_fsabilities_readwrite('archive', rbdir)
+	Log(str(fsa), 3)
+
+	update_triple(fsa.eas, ('eas_active', 'eas_write', 'eas_conn'))
+	update_triple(fsa.acls, ('acls_active', 'acls_write', 'acls_conn'))
+	update_triple(fsa.resource_forks,
+				  ('resource_forks_active', 'resource_forks_write',
+				   'resource_forks_conn'))
+	update_triple(fsa.carbonfile,
+				  ('carbonfile_active', 'carbonfile_write', 'carbonfile_conn'))
+
+	SetConnections.UpdateGlobal('preserve_hardlinks', fsa.hardlinks)
+	SetConnections.UpdateGlobal('fsync_directories', fsa.fsync_dirs)
+	SetConnections.UpdateGlobal('change_ownership', fsa.ownership)
+	SetConnections.UpdateGlobal('chars_to_quote', fsa.chars_to_quote)
+	if Globals.chars_to_quote:
+		for conn in Globals.connections:
+			conn.FilenameMapping.set_init_quote_vals()
+
 
 def ListIncrementSizes(rp):
 	"""Print out a summary of the increments """
-	assert restore_set_root(rp)
-	restore_check_backup_dir(restore_root)
+	rp = require_root_set(rp)
 	print manage.ListIncrementSizes(restore_root, restore_index)
 
 
@@ -636,7 +711,8 @@ def CalculateAverage(rps):
 
 def RemoveOlderThan(rootrp):
 	"""Remove all increment files older than a certain time"""
-	rot_check_dir(rootrp)
+	rootrp = require_root_set(rootrp)
+	rot_require_rbdir_base(rootrp)
 	try: time = Time.genstrtotime(remove_older_than_string)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
 	timep = Time.timetopretty(time)
@@ -661,41 +737,60 @@ def RemoveOlderThan(rootrp):
 	else: Log("Deleting increments at times:\n" + inc_pretty_time, 3)
 	manage.delete_earlier_than(Globals.rbdir, time)
 
-def rot_check_dir(rootrp):
-	"""Check destination dir before RemoveOlderThan"""
-	SetConnections.UpdateGlobal('rbdir',
-								rootrp.append_path("rdiff-backup-data"))
-	if not Globals.rbdir.isdir():
-		Log.FatalError("Unable to open rdiff-backup-data dir %s" %
-					   (Globals.rbdir.path,))
-	checkdest_if_necessary(rootrp)
+def rot_require_rbdir_base(rootrp):
+	"""Make sure pointing to base of rdiff-backup dir"""
+	if restore_index != ():
+		Log.FatalError("Increments for directory %s cannot be removed separately.\n"
+					   "Instead run on entire directory %s." %
+					   (rootrp.path, restore_root.path))
 
 
 def ListChangedSince(rp):
 	"""List all the files under rp that have changed since restoretime"""
-	assert restore_set_root(rp)
+	rp = require_root_set(rp)
 	try: rest_time = Time.genstrtotime(restore_timestr)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
-	restore_check_backup_dir(restore_root)
 	mirror_rp = restore_root.new_index(restore_index)
 	inc_rp = mirror_rp.append_path("increments", restore_index)
 	for rorp in rp.conn.restore.ListChangedSince(mirror_rp, inc_rp, rest_time):
-		# This is a hack, see restore.ListChangedSince for rational
+		# This is a hack, see restore.ListChangedSince for rationale
 		print rorp.index[0]
 
 
 def ListAtTime(rp):
 	"""List files in archive under rp that are present at restoretime"""
-	assert restore_set_root(rp)
+	rp = require_root_set(rp)
 	try: rest_time = Time.genstrtotime(restore_timestr)
 	except Time.TimeException, exc: Log.FatalError(str(exc))
-	restore_check_backup_dir(restore_root)
 	mirror_rp = restore_root.new_index(restore_index)
 	inc_rp = mirror_rp.append_path("increments", restore_index)
 	for rorp in rp.conn.restore.ListAtTime(mirror_rp, inc_rp, rest_time):
 		print rorp.get_indexpath()
 	
 
+def Compare(src_rp, dest_rp, compare_time = None):
+	"""Compare metadata in src_rp with metadata of backup session
+
+	Prints to stdout whenever a file in the src_rp directory has
+	different metadata than what is recorded in the metadata for the
+	appropriate session.
+
+	Session time is read from restore_timestr if compare_time is None.
+
+	"""
+	global return_val
+	dest_rp = require_root_set(dest_rp)
+	if not compare_time:
+		try: compare_time = Time.genstrtotime(restore_timestr)
+		except Time.TimeException, exc: Log.FatalError(str(exc))
+
+	mirror_rp = restore_root.new_index(restore_index)
+	inc_rp = mirror_rp.append_path("increments", restore_index)
+	backup_set_select(src_rp) # Sets source rorp iterator
+	src_iter = src_rp.conn.backup.SourceStruct.get_source_select()
+	return_val = restore.Compare(src_iter, mirror_rp, inc_rp, compare_time)
+
+
 def CheckDest(dest_rp):
 	"""Check the destination directory, """
 	if Globals.chars_to_quote:
