@@ -96,7 +96,7 @@ def set_security_level(action, cmdpairs):
 			rdir = getpath(cp2)
 	elif action in ["test-server", "list-increments", 'list-increment-sizes',
 					 "list-at-time", "list-changed-since",
-					 "calculate-average", "remove-older-than"]:
+					 "calculate-average", "remove-older-than", "compare"]:
 		sec_level = "minimal"
 		rdir = tempfile.gettempdir()
 	else: assert 0, "Unknown action %s" % action
@@ -115,6 +115,7 @@ def set_allowed_requests(sec_level):
 						"log.Log.open_logfile_allconn",
 						"log.Log.close_logfile_allconn",
 						"Log.log_to_file",
+						"FilenameMapping.set_init_quote_vals_local",
 						"SetConnections.add_redirected_conn",
 						"RedirectedRun",
 						"sys.stdout.write",
@@ -166,7 +167,6 @@ def set_allowed_requests(sec_level):
 			 "log.Log.setverbosity",
 			 "log.Log.setterm_verbosity",
 			 "Time.setprevtime_local",
-			 "FilenameMapping.set_init_quote_vals_local",
 			 "Globals.postset_regexp_local",
 			 "Globals.set_select",
 			 "backup.SourceStruct.set_session_info",
