@@ -135,7 +135,7 @@ class DestinationStruct:
 		if use_metadata:
 			rorp_iter = eas_acls.GetCombinedMetadataIter(
 				Globals.rbdir, Time.prevtime,
-				acls = Globals.read_acls, eas = Globals.read_eas)
+				acls = Globals.acls_active, eas = Globals.eas_active)
 			if rorp_iter: return rorp_iter
 		return get_iter_from_fs()
 
@@ -180,10 +180,10 @@ class DestinationStruct:
 
 	def get_one_sig(cls, dest_base_rpath, index, src_rorp, dest_rorp):
 		"""Return a signature given source and destination rorps"""
-		if (Globals.preserve_hardlinks and
-			Hardlink.islinked(src_rorp or dest_rorp)):
+		if (Globals.preserve_hardlinks and src_rorp and
+			Hardlink.islinked(src_rorp)):
 			dest_sig = rpath.RORPath(index)
-			dest_sig.flaglinked(Hardlink.get_link_index(dest_sig))
+			dest_sig.flaglinked(Hardlink.get_link_index(src_rorp))
 		elif dest_rorp:
 			dest_sig = dest_rorp.getRORPath()
 			if dest_rorp.isreg():
@@ -268,8 +268,8 @@ class CacheCollatedPostProcess:
 		self.statfileobj = statistics.init_statfileobj()
 		if Globals.file_statistics: statistics.FileStats.init()
 		metadata.MetadataFile.open_file()
-		if Globals.read_eas: eas_acls.ExtendedAttributesFile.open_file()
-		if Globals.read_acls: eas_acls.AccessControlListFile.open_file()
+		if Globals.eas_active: eas_acls.ExtendedAttributesFile.open_file()
+		if Globals.acls_active: eas_acls.AccessControlListFile.open_file()
 
 		# the following should map indicies to lists
 		# [source_rorp, dest_rorp, changed_flag, success_flag, increment]
@@ -312,8 +312,8 @@ class CacheCollatedPostProcess:
 		will be backed up correctly.
 
 		"""
-		if source_rorp: Hardlink.add_rorp(source_rorp, source = 1)
-		if dest_rorp: Hardlink.add_rorp(dest_rorp, source = 0)
+		if Globals.preserve_hardlinks and source_rorp:
+			Hardlink.add_rorp(source_rorp, dest_rorp)
 		if (dest_rorp and dest_rorp.isdir() and Globals.process_uid != 0
 			and dest_rorp.getperms() % 01000 < 0700):
 			self.unreadable_dir_init(source_rorp, dest_rorp)
@@ -355,6 +355,9 @@ class CacheCollatedPostProcess:
 		always false for un-changed files).
 
 		"""
+		if Globals.preserve_hardlinks and source_rorp:
+			Hardlink.del_rorp(source_rorp)
+
 		if not changed or success:
 			if source_rorp: self.statfileobj.add_source_file(source_rorp)
 			if dest_rorp: self.statfileobj.add_dest_file(dest_rorp)
@@ -365,10 +368,10 @@ class CacheCollatedPostProcess:
 		else: metadata_rorp = None
 		if metadata_rorp and metadata_rorp.lstat():
 			metadata.MetadataFile.write_object(metadata_rorp)
-			if Globals.read_eas and not metadata_rorp.get_ea().empty():
+			if Globals.eas_active and not metadata_rorp.get_ea().empty():
 				eas_acls.ExtendedAttributesFile.write_object(
 					metadata_rorp.get_ea())
-			if Globals.read_acls and not metadata_rorp.get_acl().is_basic():
+			if Globals.acls_active and not metadata_rorp.get_acl().is_basic():
 				eas_acls.AccessControlListFile.write_object(
 					metadata_rorp.get_acl())
 		if Globals.file_statistics:
@@ -424,8 +427,8 @@ class CacheCollatedPostProcess:
 			dir_rp, perms = self.dir_perms_list.pop()
 			dir_rp.chmod(perms)
 		metadata.MetadataFile.close_file()
-		if Globals.read_eas: eas_acls.ExtendedAttributesFile.close_file()
-		if Globals.read_acls: eas_acls.AccessControlListFile.close_file()
+		if Globals.eas_active: eas_acls.ExtendedAttributesFile.close_file()
+		if Globals.acls_active: eas_acls.AccessControlListFile.close_file()
 		if Globals.print_statistics: statistics.print_active_stats()
 		if Globals.file_statistics: statistics.FileStats.close()
 		statistics.write_active_statfileobj()
