@@ -248,7 +248,7 @@ or within one second of each other.  Wait a second and try again.""" %
 		"""Call cls.write(...) if error log open, only log otherwise"""
 		if not Globals.isbackup_writer:
 			return Globals.backup_writer.log.ErrorLog.write_if_open(
-				error_type, rp, exc)
+				error_type, rp, str(exc)) # convert exc bc of exc picking prob
 		if cls.isopen(): cls.write(error_type, rp, exc)
 		else: Log(cls.get_log_string(error_type, rp, exc), 2)
 
