@@ -1,4 +1,4 @@
-# Copyright 2002, 2003 Ben Escoto
+# Copyright 2002, 2003, 2004 Ben Escoto
 #
 # This file is part of rdiff-backup.
 #
@@ -25,11 +25,6 @@ import Globals, Time, Rdiff, Hardlink, rorpiter, selection, rpath, \
 	   log, static, robust, metadata, statistics, TempFile, eas_acls
 
 
-# This should be set to selection.Select objects over the source and
-# mirror directories respectively.
-_select_source = None
-_select_mirror = None
-
 # This will be set to the time of the current mirror
 _mirror_time = None
 # This will be set to the exact time to restore to (not restore_to_time)
@@ -103,6 +98,31 @@ def ListAtTime(mirror_rp, inc_rp, time):
 	old_iter = MirrorStruct.get_mirror_rorp_iter(_rest_time, 1)
 	for rorp in old_iter: yield rorp
 	
+def Compare(src_iter, mirror_rp, inc_rp, compare_time):
+	"""Compares metadata in src_rp dir with metadata in mirror_rp at time"""
+	MirrorStruct.set_mirror_and_rest_times(compare_time)
+	MirrorStruct.initialize_rf_cache(mirror_rp, inc_rp)
+
+	mir_iter = MirrorStruct.get_mirror_rorp_iter(compare_time, 1)
+	collated = rorpiter.Collate2Iters(src_iter, mir_iter)
+	changed_files_found = 0
+	for src_rorp, mir_rorp in collated: 
+		if not mir_rorp: change = "new"
+		elif not src_rorp: change = "deleted"
+		elif src_rorp == mir_rorp: continue
+		else: change = "changed"
+		changed_files_found = 1
+		path_desc = (src_rorp and src_rorp.get_indexpath() or
+					 mir_rorp.get_indexpath())
+		log.Log("%-7s %s" % (change, path_desc), 2)
+		if change == "changed": # Log more description of difference
+			assert not src_rorp.equal_verbose_auto(mir_rorp, 3)
+
+	if not changed_files_found:
+		log.Log("No changes found.  Directory matches archive data.", 2)
+	MirrorStruct.close_rf_cache()
+	return changed_files_found
+
 
 class MirrorStruct:
 	"""Hold functions to be run on the mirror side"""
@@ -263,9 +283,16 @@ static.MakeClass(MirrorStruct)
 
 class TargetStruct:
 	"""Hold functions to be run on the target side when restoring"""
-	def get_initial_iter(cls, target):
+	_select = None
+	def set_target_select(cls, target, select_opts, *filelists):
 		"""Return a selection object iterating the rorpaths in target"""
-		return selection.Select(target).set_iter()
+		cls._select = selection.Select(target)
+		cls._select.ParseArgs(select_opts, filelists)
+		cls._select.set_iter()
+
+	def get_initial_iter(cls, target):
+		"""Return selector previously set with set_initial_iter"""
+		return cls._select or selection.Select(target).set_iter()
 
 	def patch(cls, target, diff_iter):
 		"""Patch target with the diffs from the mirror side
@@ -525,29 +552,39 @@ as data loss may result.\n""" % (self.mirror_rp.get_indexpath(),), 2)
 
 		"""
 		if not inc_rpath.isdir(): return
-		inc_dict = {} # dictionary of basenames:IndexedTuples(index, inc_list)
-		dirlist = robust.listrp(inc_rpath)
 
-		def affirm_dict_indexed(basename):
-			"""Make sure the rid dictionary has given basename as key"""
-			if not inc_dict.has_key(basename):
-				sub_inc_rp = inc_rpath.append(basename)
-				inc_dict[basename] = rorpiter.IndexedTuple(sub_inc_rp.index,
-														   (sub_inc_rp, []))
+		def get_inc_pairs():
+			"""Return unsorted list of (basename, inc_filenames) pairs"""
+			inc_dict = {} # dictionary of basenames:inc_filenames
+			dirlist = robust.listrp(inc_rpath)
 
-		def add_to_dict(filename):
-			"""Add filename to the inc tuple dictionary"""
-			rp = inc_rpath.append(filename)
-			if rp.isincfile() and rp.getinctype() != 'data':
-				basename = rp.getincbase_str()
-				affirm_dict_indexed(basename)
-				inc_dict[basename][1].append(rp)
-			elif rp.isdir(): affirm_dict_indexed(filename)
+			def add_to_dict(filename):
+				"""Add filename to the inc tuple dictionary"""
+				rp = inc_rpath.append(filename)
+				if rp.isincfile() and rp.getinctype() != 'data':
+					basename = rp.getincbase_str()
+					inc_filename_list = inc_dict.setdefault(basename, [])
+					inc_filename_list.append(filename)
+				elif rp.isdir(): inc_dict.setdefault(filename, [])
 
-		for filename in dirlist: add_to_dict(filename)
-		keys = inc_dict.keys()
-		keys.sort()
-		for key in keys: yield inc_dict[key]
+			for filename in dirlist: add_to_dict(filename)
+			return inc_dict.items()
+
+		def inc_filenames2incrps(filenames):
+			"""Map list of filenames into increment rps"""
+			l = []
+			for filename in filenames:
+				rp = inc_rpath.append(filename)
+				assert rp.isincfile(), rp.path
+				l.append(rp)
+			return l
+
+		items = get_inc_pairs()
+		items.sort() # Sorting on basis of basename now
+		for (basename, inc_filenames) in items:
+			sub_inc_rpath = inc_rpath.append(basename)
+			yield rorpiter.IndexedTuple(sub_inc_rpath.index,
+					  (sub_inc_rpath, inc_filenames2incrps(inc_filenames)))
 
 
 class PatchITRB(rorpiter.ITRBranch):
