@@ -1,4 +1,4 @@
-# Copyright 2002, 2003 Ben Escoto
+# Copyright 2002, 2003, 2004 Ben Escoto
 #
 # This file is part of rdiff-backup.
 #
@@ -155,6 +155,8 @@ def copy_attribs(rpin, rpout):
 	if rpin.issym(): return # symlinks have no valid attributes
 	if Globals.resource_forks_write and rpin.isreg():
 		rpout.write_resource_fork(rpin.get_resource_fork())
+	if Globals.carbonfile_write and rpin.isreg():
+		rpout.write_carbonfile(rpin.get_carbonfile())
 	if Globals.eas_write: rpout.write_ea(rpin.get_ea())
 	if Globals.change_ownership: rpout.chown(*user_group.map_rpath(rpin))
 	rpout.chmod(rpin.getperms())
@@ -174,6 +176,8 @@ def copy_attribs_inc(rpin, rpout):
 	if rpin.issym(): return # symlinks have no valid attributes
 	if Globals.resource_forks_write and rpin.isreg() and rpout.isreg():
 		rpout.write_resource_fork(rpin.get_resource_fork())
+	if Globals.carbonfile_write and rpin.isreg() and rpout.isreg():
+		rpout.write_carbonfile(rpin.get_carbonfile())
 	if Globals.eas_write: rpout.write_ea(rpin.get_ea())
 	if Globals.change_ownership: apply(rpout.chown, rpin.getuidgid())
 	if rpin.isdir() and not rpout.isdir():
@@ -294,11 +298,14 @@ class RORPath:
 			elif key == 'size' and not self.isreg(): pass
 			elif key == 'ea' and not Globals.eas_active: pass
 			elif key == 'acl' and not Globals.acls_active: pass
+			elif key == 'carbonfile' and not Globals.carbonfile_active: pass
 			elif key == 'resourcefork' and not Globals.resource_forks_active:
 				pass
-			elif ((key == 'uname' or key == 'gname') and
-				  not other.data.has_key(key)):
-				pass # legacy reasons - 0.12.x didn't store u/gnames
+			elif key == 'uname' or key == 'gname':
+				# here for legacy reasons - 0.12.x didn't store u/gnames
+				other_name = other.data.get(key, None)
+				if (other_name and other_name != "None" and
+					other_name != self.data[key]): return None
 			elif (key == 'inode' and
 				  (not self.isreg() or self.getnumlinks() == 1 or
 				   not Globals.compare_inode or
@@ -307,7 +314,7 @@ class RORPath:
 			else:
 				try: other_val = other.data[key]
 				except KeyError: return None
-				if self.data[key] != other.data[key]: return None
+				if self.data[key] != other_val: return None
 		return 1
 
 	def equal_loose(self, other):
@@ -331,6 +338,7 @@ class RORPath:
 			elif key == 'inode': pass
 			elif key == 'ea' and not Globals.eas_write: pass
 			elif key == 'acl' and not Globals.acls_write: pass
+			elif key == 'carbonfile' and not Globals.carbonfile_write: pass
 			elif key == 'resourcefork' and not Globals.resource_forks_write:
 				pass
 			elif (not other.data.has_key(key) or
@@ -344,10 +352,11 @@ class RORPath:
 
 	def equal_verbose(self, other, check_index = 1,
 					  compare_inodes = 0, compare_ownership = 0,
-					  compare_acls = 0, compare_eas = 0):
+					  compare_acls = 0, compare_eas = 0, verbosity = 2):
 		"""Like __eq__, but log more information.  Useful when testing"""
 		if check_index and self.index != other.index:
-			log.Log("Index %s != index %s" % (self.index, other.index), 2)
+			log.Log("Index %s != index %s" % (self.index, other.index),
+					verbosity)
 			return None
 
 		for key in self.data.keys(): # compare dicts key by key
@@ -366,12 +375,22 @@ class RORPath:
 			elif (not other.data.has_key(key) or
 				  self.data[key] != other.data[key]):
 				if not other.data.has_key(key):
-					log.Log("Second is missing key %s" % (key,), 2)
+					log.Log("Second is missing key %s" % (key,), verbosity)
 				else: log.Log("Value of %s differs: %s vs %s" %
-							  (key, self.data[key], other.data[key]), 2)
+							  (key, self.data[key], other.data[key]),
+							  verbosity)
 				return None
 		return 1
 
+	def equal_verbose_auto(self, other, verbosity = 2):
+		"""Like equal_verbose, but set parameters like __eq__ does"""
+		compare_inodes = ((self.getnumlinks() != 1) and
+						  Globals.compare_inode and Globals.preserve_hardlinks)
+		return self.equal_verbose(other,
+								  compare_inodes = compare_inodes,
+								  compare_eas = Globals.eas_active,
+								  compare_acls = Globals.acls_active)
+							 
 	def __ne__(self, other): return not self.__eq__(other)
 
 	def __str__(self):
@@ -587,6 +606,18 @@ class RORPath:
 		"""Return extended attributes object"""
 		return self.data['ea']
 
+	def has_carbonfile(self):
+		"""True if rpath has a carbonfile parameter"""
+		return self.data.has_key('carbonfile')
+
+	def get_carbonfile(self):
+		"""Returns the carbonfile data"""
+		return self.data['carbonfile']
+
+	def set_carbonfile(self, cfile):
+		"""Record carbonfile data in dictionary.  Does not write."""
+		self.data['carbonfile'] = cfile
+
 	def has_resource_fork(self):
 		"""True if rpath has a resourcefork parameter"""
 		return self.data.has_key('resourcefork')
@@ -1057,6 +1088,41 @@ class RPath(RORPath):
 		ea.write_to_rp(self)
 		self.data['ea'] = ea
 
+	def get_carbonfile(self):
+		"""Return resource fork data, loading from filesystem if
+		necessary."""
+		from Carbon.File import FSSpec
+		import MacOS
+		try: return self.data['cfile']
+		except KeyError: pass
+
+		try:
+			fsobj = FSSpec(self.path)
+			finderinfo = fsobj.FSpGetFInfo()
+			cfile = {'creator': finderinfo.Creator,
+					 'type': finderinfo.Type,
+					 'location': finderinfo.Location,
+					 'flags': finderinfo.Flags}
+			self.data['carbonfile'] = cfile
+			return cfile
+		except MacOS.Error:
+			self.data['carbonfile'] = None 
+			return self.data['carbonfile']
+
+	def write_carbonfile(self, cfile):
+		"""Write new carbon data to self."""
+		log.Log("Writing carbon data to %s" % (self.index,), 7)
+		from Carbon.File import FSSpec
+		import MacOS
+		fsobj = FSSpec(self.path)
+		finderinfo = fsobj.FSpGetFInfo()
+		finderinfo.Creator = cfile['creator']
+		finderinfo.Type = cfile['type']
+		finderinfo.Location = cfile['location']
+		finderinfo.Flags = cfile['flags']
+		fsobj.FSpSetFInfo(finderinfo)
+		self.set_carbonfile(cfile)
+
 	def get_resource_fork(self):
 		"""Return resource fork data, setting if necessary"""
 		assert self.isreg()
@@ -1110,7 +1176,7 @@ def setdata_local(rpath):
 	if Globals.acls_conn: rpath.data['acl'] = acl_get(rpath)
 	if Globals.resource_forks_conn and rpath.isreg():
 		rpath.get_resource_fork()
-
+	if Globals.carbonfile_conn and rpath.isreg(): rpath.get_carbonfile()
 
 # These two are overwritten by the eas_acls.py module.  We can't
 # import that module directly because of circular dependency problems.
